@@ -92,7 +92,11 @@ void SetMessageVariables(const FieldDescriptor* descriptor,
   const JavaType valueJavaType = GetJavaType(value);
 
   (*variables)["key_type"] = TypeName(key, name_resolver, false);
-  (*variables)["boxed_key_type"] = TypeName(key, name_resolver, true);
+  string boxed_key_type = TypeName(key, name_resolver, true);
+  (*variables)["boxed_key_type"] = boxed_key_type;
+  // Used for calling the serialization function.
+  (*variables)["short_key_type"] =
+      boxed_key_type.substr(boxed_key_type.rfind('.') + 1);
   (*variables)["key_wire_type"] = WireType(key);
   (*variables)["key_default_value"] = DefaultValue(key, true, name_resolver);
   (*variables)["key_null_check"] = IsReferenceType(keyJavaType) ?
@@ -147,6 +151,7 @@ void SetMessageVariables(const FieldDescriptor* descriptor,
       name_resolver->GetImmutableClassName(descriptor->file()) +
       ".internal_" + UniqueFileScopeIdentifier(descriptor->message_type()) +
       "_descriptor, ";
+  (*variables)["ver"] = GeneratedCodeVersionSuffix();
 }
 
 }  // namespace
@@ -661,21 +666,23 @@ GenerateParsingCode(io::Printer* printer) const {
         variables_,
         "com.google.protobuf.ByteString bytes = input.readBytes();\n"
         "com.google.protobuf.MapEntry<$type_parameters$>\n"
-        "$name$ = $default_entry$.getParserForType().parseFrom(bytes);\n");
+        "$name$__ = $default_entry$.getParserForType().parseFrom(bytes);\n");
     printer->Print(
         variables_,
-        "if ($value_enum_type$.forNumber($name$.getValue()) == null) {\n"
+        "if ($value_enum_type$.forNumber($name$__.getValue()) == null) {\n"
         "  unknownFields.mergeLengthDelimitedField($number$, bytes);\n"
         "} else {\n"
-        "  $name$_.getMutableMap().put($name$.getKey(), $name$.getValue());\n"
+        "  $name$_.getMutableMap().put(\n"
+        "      $name$__.getKey(), $name$__.getValue());\n"
         "}\n");
   } else {
     printer->Print(
         variables_,
         "com.google.protobuf.MapEntry<$type_parameters$>\n"
-        "$name$ = input.readMessage(\n"
+        "$name$__ = input.readMessage(\n"
         "    $default_entry$.getParserForType(), extensionRegistry);\n"
-        "$name$_.getMutableMap().put($name$.getKey(), $name$.getValue());\n");
+        "$name$_.getMutableMap().put(\n"
+        "    $name$__.getKey(), $name$__.getValue());\n");
   }
 }
 
@@ -688,15 +695,12 @@ void ImmutableMapFieldGenerator::
 GenerateSerializationCode(io::Printer* printer) const {
   printer->Print(
       variables_,
-      "for (java.util.Map.Entry<$type_parameters$> entry\n"
-      "     : internalGet$capitalized_name$().getMap().entrySet()) {\n"
-      "  com.google.protobuf.MapEntry<$type_parameters$>\n"
-      "  $name$ = $default_entry$.newBuilderForType()\n"
-      "      .setKey(entry.getKey())\n"
-      "      .setValue(entry.getValue())\n"
-      "      .build();\n"
-      "  output.writeMessage($number$, $name$);\n"
-      "}\n");
+      "com.google.protobuf.GeneratedMessage$ver$\n"
+      "  .serialize$short_key_type$MapTo(\n"
+      "    output,\n"
+      "    internalGet$capitalized_name$(),\n"
+      "    $default_entry$,\n"
+      "    $number$);\n");
 }
 
 void ImmutableMapFieldGenerator::
@@ -706,12 +710,12 @@ GenerateSerializedSizeCode(io::Printer* printer) const {
       "for (java.util.Map.Entry<$type_parameters$> entry\n"
       "     : internalGet$capitalized_name$().getMap().entrySet()) {\n"
       "  com.google.protobuf.MapEntry<$type_parameters$>\n"
-      "  $name$ = $default_entry$.newBuilderForType()\n"
+      "  $name$__ = $default_entry$.newBuilderForType()\n"
       "      .setKey(entry.getKey())\n"
       "      .setValue(entry.getValue())\n"
       "      .build();\n"
       "  size += com.google.protobuf.CodedOutputStream\n"
-      "      .computeMessageSize($number$, $name$);\n"
+      "      .computeMessageSize($number$, $name$__);\n"
       "}\n");
 }
 
