@@ -35,10 +35,14 @@ internal_build_cpp() {
 build_cpp() {
   internal_build_cpp
   make check -j2
-  cd conformance && make test_cpp && cd ..
+  pushd conformance
+  make test_cpp
+  popd
 
   # Verify benchmarking code can build successfully.
-  cd benchmarks && make && ./generate-datasets && cd ..
+  pushd benchmarks
+  make && ./generate-datasets
+  popd
 }
 
 build_cpp_distcheck() {
@@ -47,7 +51,7 @@ build_cpp_distcheck() {
   make dist
 
   # List all files that should be included in the distribution package.
-  git ls-files | grep "^\(java\|python\|objectivec\|csharp\|js\|ruby\|cmake\|examples\)" |\
+  git ls-files | grep "^\(java\|python\|objectivec\|csharp\|js\|ruby\|php\|cmake\|examples\)" |\
       grep -v ".gitignore" | grep -v "java/compatibility_tests" > dist.lst
   # Unzip the dist tar file.
   DIST=`ls *.tar.gz`
@@ -195,6 +199,14 @@ build_java_oracle7() {
   use_java oracle7
   build_java oracle7
 }
+build_java_compatibility() {
+  use_java jdk7
+  internal_build_cpp
+  # Use the unit-tests extraced from 2.5.0 to test the compatibilty between
+  # 3.0.0-beta-4 and the current version.
+  cd java/compatibility_tests/v2.5.0
+  ./test.sh 3.0.0-beta-4
+}
 
 build_javanano_jdk7() {
   use_java jdk7
@@ -284,8 +296,6 @@ build_python() {
 build_python_cpp() {
   internal_build_cpp
   internal_install_python_deps
-  export LD_LIBRARY_PATH=../src/.libs # for Linux
-  export DYLD_LIBRARY_PATH=../src/.libs # for OS X
   cd python
   # Only test Python 2.6/3.x on Linux
   if [ $(uname -s) == "Linux" ]; then
@@ -340,6 +350,7 @@ Usage: $0 { cpp |
             csharp |
             java_jdk7 |
             java_oracle7 |
+            java_compatibility |
             javanano_jdk7 |
             javanano_oracle7 |
             objectivec_ios |
